'use strict';

var React = require('react');

var PureComponent = React.PureComponent;
var h = React.createElement;

function extend(superClass, obj, props) {
    function __() {
        this.constructor = obj;
    }
    __.prototype = superClass.prototype;
    obj.prototype = new __();
    for (var i in props) obj.prototype[i] = props[i];
}

var wrap = function (Context, key, WrappedComp) {
    function Connected(props) {
        PureComponent.call(this, props);
    }

    extend(PureComponent, Connected, {
        render() {
            const props = this.props;
            return h(Context.Consumer, null, function (context) {
                var tmpProps = {};
                tmpProps[key] = context;
                return h(WrappedComp, Object.assign(tmpProps, props));
            });
        }
    });

    Connected.WrappedComp = WrappedComp;
    var wrappedCompName = WrappedComp.displayName || WrappedComp.name || "Component";
    var consumerName = Context.Consumer.displayName || Context.Consumer.name || "Context.Consumer";
    Connected.displayName = `${wrappedCompName}(${consumerName}.${key})`;

    return Connected;
};

function withContext(Context, key = "context") {
    return function (WrappedComp) {
        return wrap(Context, key, WrappedComp);
    };
}

function withMultiContext(map) {
    return function (WrappedComp) {
        return Object.keys(map).reduce(function (Comp, key) {
            return wrap(map[key], key, Comp);
        }, WrappedComp);
    };
}

module.exports.withContext = withContext;
module.exports.withMultiContext = withMultiContext;